/*
//@line 38 "e:\builds\moz2_slave\w32mob-addonsmgr-nightly\addonsmgr\toolkit\mozapps\extensions\extManager.js"
*/

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const PREF_EM_UPDATE_INTERVAL         = "extensions.update.interval";
const XPI_CONTENT_TYPE                = "application/x-xpinstall";

const UNSUPPORTED_TYPE = -244;
const CANT_READ_ARCHIVE = -207;
const USER_CANCELLED = -210;
const DOWNLOAD_ERROR = -228;
const SUCCESS = 0;

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

XPCOMUtils.defineLazyGetter(this, "Services", function() {
  let scope = {};
  Components.utils.import("resource://gre/modules/Services.jsm", scope);
  return scope.Services;
});

var gSingleton = null;

function extManager() {
  Components.utils.import("resource://gre/modules/AddonManager.jsm");
}

extManager.prototype = {
  observe: function AMC_observe(subject, topic, data) {
    var os = Cc["@mozilla.org/observer-service;1"].
             getService(Ci.nsIObserverService);

    switch (topic) {
    case "profile-after-change":
      os.addObserver(this, "xpcom-shutdown", false);
      AddonManagerInternal.startup();
      break;
    case "xpcom-shutdown":
      os.removeObserver(this, "xpcom-shutdown");
      AddonManagerInternal.shutdown();
      break;
    }
  },

  get needsRestart() {
    return AddonManagerInternal.needsRestart;
  },

  isInstallEnabled: function AMC_isInstallEnabled(type) {
    return AddonManager.isInstallEnabled(type);
  },

  startInstall: function AMC_startInstall(type, window, referer, uris, hashes,
                                          names, icons, callback) {
    if (uris.length == 0)
      return false;

    let retval = true;
    if (!AddonManager.isInstallAllowed(type, referer)) {
      callback = null;
      retval = false;
    }

    let loadgroup = null;
    try {
      if (window) {
        loadgroup = window.QueryInterface(Ci.nsIInterfaceRequestor)
                          .getInterface(Ci.nsIWebNavigation)
                          .QueryInterface(Ci.nsIDocumentLoader).loadGroup;
      }
    }
    catch (e) {
    }

    let installs = [];
    function buildNextInstall() {
      if (uris.length == 0) {
        AddonManager.startInstallation(type, window, referer, installs, true);
        return;
      }
      let uri = uris.shift();
      AddonManager.getInstallForURL(uri, function(install) {
        if (install) {
          installs.push(install);
          if (callback) {
            install.addListener({
              onDownloadCancelled: function(install) {
                callback.callback(uri, USER_CANCELLED);
              },

              onDownloadFailed: function(install, error) {
                if (error == AddonManager.DOWNLOADERROR_CORRUPT_FILE)
                  callback.callback(uri, CANT_READ_ARCHIVE);
                else
                  callback.callback(uri, DOWNLOAD_ERROR);
              },

              onInstallFailed: function(install, error) {
              },

              onInstallEnded: function(install, status) {
                callback.callback(uri, SUCCESS);
              }
            });
          }
        }
        else if (callback) {
          callback.callback(uri, UNSUPPORTED_TYPE);
        }
        buildNextInstall();
      }, type, hashes.shift(), names.shift(), icons.shift(), null, loadgroup);
    }
    buildNextInstall();

    return retval;
  },

  notify: function EM_notify(timer) {
    AddonManagerInternal.backgroundUpdateCheck();
  },

  classDescription: "Addons Manager",
  contractID: "@mozilla.org/extensions/manager;2",
  classID: Components.ID("{4399533d-08d1-458c-a87a-235f74451cfa}"),
  _xpcom_categories: [{ category: "profile-after-change" },
                      { category: "update-timer",
                        value: "@mozilla.org/extensions/manager;2," +
                               "getService,addon-background-update-timer," +
                               PREF_EM_UPDATE_INTERVAL + ",86400" }],
  _xpcom_factory: {
    createInstance: function(outer, iid) {
      if (outer != null)
        throw Cr.NS_ERROR_NO_AGGREGATION;
  
      if (!gSingleton)
        gSingleton = new extManager();
      return gSingleton.QueryInterface(iid);
    }
  },
  QueryInterface: XPCOMUtils.generateQI([Ci.extIManager,
                                         Ci.extIManagerInternal,
                                         Ci.nsITimerCallback,
                                         Ci.nsIObserver])
};

function extXPIContentHandler() {
}

extXPIContentHandler.prototype = {
  handleContent: function XCH_handleContent(type, context, request) {
    if (type != XPI_CONTENT_TYPE)
      throw Cr.NS_ERROR_WONT_HANDLE_CONTENT;

    if (!(request instanceof Ci.nsIChannel))
      throw Cr.NS_ERROR_WONT_HANDLE_CONTENT;

    let uri = request.URI;

    let referer = null;
    if (request instanceof Ci.nsIPropertyBag2)
      referer = request.getPropertyAsInterface("docshell.internalReferrer", Ci.nsIURI);

    let window = null;
    let callbacks = request.notificationCallbacks ?
                    request.notificationCallbacks :
                    request.loadGroup.notificationCallbacks;
    if (callbacks)
      window = callbacks.getInterface(Ci.nsIDOMWindow);

    request.cancel(Cr.NS_BINDING_ABORTED);

    let manager = Cc["@mozilla.org/extensions/manager;2"].getService(Ci.extIManager);
    manager.startInstall(type, window, referer, [uri.spec], [null], [null], [null], null, 1);
  },

  classDescription: "XPI Content Handler",
  contractID: "@mozilla.org/uriloader/content-handler;1?type=" + XPI_CONTENT_TYPE,
  classID: Components.ID("{7beb3ba8-6ec3-41b4-b67c-da89b8518922}"),
  QueryInterface: XPCOMUtils.generateQI([Ci.nsIContentHandler])
};

function NSGetModule(compMgr, fileSpec)
  XPCOMUtils.generateModule([extManager, extXPIContentHandler]);
